/***********************************************************
 * $Id: fancyqt4browserframeagent.h 2090 2020-06-04 22:35:08Z cl_yubikey_2018a $
 *
 * FancyQT4 Widgets
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2016 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYQT4WIDGETS_BROWSERFRAME_AGENT_H_
#define FANCYQT4WIDGETS_BROWSERFRAME_AGENT_H_

#include <fancyqt4widgets.h>

#include <QtCore/QObject>
#include <QtCore/QUrl>

//#include <QtGui/QPrinter>
class QPrinter;

namespace Fancy {
namespace Qt4 {
namespace Widgets {

/**
 * Attempt to generalize providing a browser frame in a QT app.
 *
 * In Linux Webkit shall be used.
 *
 *
 * In Windows IE's ActiveX component Trident shall be used.
 *
 * There are 2 ways to specifically request specific IE engine,
 * both are based on the registry hacks described on
 * https://weblog.west-wind.com/posts/2011/may/21/web-browser-control-specifying-the-ie-version
 *
 * 1. specify browserTye AutoSelect or IEDefault and set a registry key on your own
 * 2. specify a particular browserTye IE*, having the BrowserFrameAgent instance set the registry key dynamically
 *
 * Possible registry keys are:
 * For current user, also writable by current user:
 * HKEY_CURRENT_USER\SOFTWARE\Microsoft\Internet Explorer\Main\FeatureControl\FEATURE_BROWSER_EMULATION
 *
 * System wide:
 * HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Internet Explorer\MAIN\FeatureControl\FEATURE_BROWSER_EMULATION
 *
 * System wide for 32 bit apps on 64 bit Windows:
 * HKEY_LOCAL_MACHINE\SOFTWARE\WOW6432Node\Microsoft\Internet Explorer\Main\FeatureControl\FEATURE_BROWSER_EMULATION
 *
 * Registry keyname: Name of your exe, i.e. fancyqt4widgets-test.exe
 *
 * Registry value: See https://msdn.microsoft.com/en-us/library/ee330730(v=vs.85).aspx#browser_emulation
 *  7000 (0x1B58): IE7
 * 10000 (0x2710): IE10Hard, enforce IE 10 no matter what
 * 10001 (0x2711): IE10Soft, use IE 11 if DOCTYPE satisfies
 * 11000 (0x2AF8): IE11Soft, use IE 11 if DOCTYPE satisfies
 * 11001 (0x2AF9): IE11Hard, enforce IE 11 no matter what
 * If the site to display is under your control, there are 2 more options to invite IE11 behaviour:
 * HTML Header:
 *   <meta http-equiv="X-UA-Compatible" content="IE=edge" />
 * HTTP Response Header:
 *   X_UA_Compatible: IE=edge
 */
class FANCYQT4WIDGETS_API BrowserFrameAgent : public QObject {

	Q_OBJECT

	//Q_PROPERTY( int maxLength READ getMaxLength WRITE setMaxLength )

	//Q__ENUMS( Font )
	// we don't care about Qt properties too much, and don't want to mirror Drawing::Font for eternity
	//Q_PROPERTY( Drawing::Font printFont READ getPrintFont WRITE setPrintFont )

public:

	enum BrowserType {
		AutoSelect = 0,
		Webkit = 1, // only available in linux
		IE7 = 2,
		IE11Soft = 3,
		IE11Hard = 4,
		IE10Soft = 5,
		IE10Hard = 6
	};

	//BrowserFrameAgent();
	virtual ~BrowserFrameAgent();

	static BrowserFrameAgent* createInstance();

	virtual QWidget* createWebWidget(
			QWidget *parentWidget=0,
			BrowserFrameAgent::BrowserType browserType=AutoSelect) = 0;
	virtual QWidget* getWebWidget() = 0;

	static std::string describeBrowserType(BrowserFrameAgent::BrowserType browserType);

	/*
	 * For some BrowserTypes (currently Webkit only) it is possible to decide
	 * if the user shall be asked to continue or abort long-running scripts.
	 *
	 * Call before createWebWidget()!
	 *
	 * @param jsTimeoutPolicy  -1: always abort, 0: let browser ask user, 1: always continue
	 */
	virtual void setJavaScriptTimeoutPolicy(int jsTimeoutPolicy) = 0;

public slots:

    virtual void browseToUrl(const QString & url) = 0;
    virtual void browseToUrl(const QUrl & url) = 0;

    virtual void back() = 0;
    virtual void forward() = 0;
    //virtual void print(QPrinter * printer) const = 0;
    virtual void reload() = 0;
    virtual void stop() = 0;

signals:

    // avoid conflicts here!
    void browserLoadProgress ( int progressPercent ); // win: 2 ints
    void browserTitleChanged ( const QString & title );

};


} /* namespace Widgets */
} /* namespace Qt4 */
} /* namespace Fancy */

#endif /* FANCYQT4WIDGETS_BROWSERFRAME_AGENT_H_ */
